import yaml
from os import path, pardir


class Config(object):

    """
    Supply system configuration

    Examples
    --------
    Get system configuration

    >>> from system.conf import Config
    >>> config = Config()
    >>> database = config["database"]
    >>> "mysql" in database
    True
    """

    __obj = ""

    def __new__(self, *args, **kwargs):
        """ Return ConfigParser """


        if kwargs:
            config_file = kwargs["config"]
        else:
            config_file = "services.cfg"
        
        if not self.__obj:
            curdir = path.dirname(path.abspath(__file__))
            config = path.join(curdir, pardir, config_file)
            with open(path.abspath(config)) as f:
                self.__obj = yaml.load(f)

        return self.__obj



from system.app import *
from system.log import Log


class LogService(App):

    """
    Support all syslog.

    >>> from nameko.standalone.rpc import ClusterRpcProxy
    >>> CONFIG = {"AMQP_URI":"amqp://guest:guest@localhost"}
    >>> with ClusterRpcProxy(CONFIG) as services:
    >>>     result = services.log_service.add_log({"content": "info"})
    """

    name = "log_service"

    def __init__(self):
        super(LogService, self).init()

    @rpc
    def add_log(self, content):
        """ Add a log. """
        
        self.log = Log("sysLog").get()
        result = self.log.insert_one(content)
        return result.acknowledged



from sqlalchemy import Column, Integer, String, create_engine, types
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker
from system.conf import Config
from redis import StrictRedis
from sqlalchemy import and_, or_, distinct, func

Base = declarative_base()
BaseRegion = declarative_base()
BaseFiling = declarative_base()


class Db(object):

    """
    Create system db mechanics

    Examples
    --------
    Create database and datasets

    >>> from system.db import *
    >>> _db = Db()
    >>> db = _db.get_db()
    >>> isinstance(db, scoped_session)
    True
    >>> rs = _db.get_rs()
    >>> isinstance(rs, StrictRedis)
    True
    """

    def __init__(self):
        self.config = Config()

    def get_db(self):
        """
        Get a session
        For others, it's db.
        """

        self.engine = create_engine(self.config["database"])
        Base.metadata.create_all(self.engine)
        return scoped_session(sessionmaker(self.engine))

    def drop_db(self):
        """  Drop all db.  """

        Base.metadata.drop_all(self.engine)

    def get_rs(self):
        """ Support redis """

        return StrictRedis.from_url(self.config["datasets"])



from system.db import *


class ConnEvents(Base):
    __tablename__ = 'conn_events'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    conn_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    user_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    ip = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    category = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    action = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    data1 = Column(types.Unicode(1024), nullable=True, unique=None, default=u'')
    data2 = Column(types.Unicode(1024), nullable=True, unique=None, default=u'')
    data3 = Column(types.Unicode(1024), nullable=True, unique=None, default=u'')
    data4 = Column(types.Unicode(1024), nullable=True, unique=None, default=u'')
    data5 = Column(types.Unicode(1024), nullable=True, unique=None, default=u'')
    time_recorded = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    time_updated = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    time_recorded_int = Column(types.Integer(), nullable=True, unique=None, default=0)
    time_updated_int = Column(types.Integer(), nullable=True, unique=None, default=0)
    state = Column(types.Unicode(255), nullable=True, unique=None, default=u'open')


class MachineStockEvents(Base):
    __tablename__ = 'machine_stock_events'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    id = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    client_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    machine_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    machine_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    slot_id = Column(types.Unicode(45), nullable=True, unique=None, default=u'')
    product_id = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    sku = Column(types.Unicode(45), nullable=True, unique=None, default=u'')
    upc = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    fina_cate_id = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    fina_cate_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    unit_price = Column(types.Numeric(precision=20, scale=3), nullable=True, unique=None, default=0.000)
    quantity = Column(types.Integer(), nullable=True, unique=None, default=0)
    action_time = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    action_time_int = Column(types.Integer(), nullable=True, unique=None, default=0)
    type = Column(types.Unicode(45), nullable=True, unique=None, default=u'')
    notes = Column(types.UnicodeText(), nullable=True, unique=None, default=None)
    receipt_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    operator = Column(types.Unicode(255), nullable=True, unique=None, default=u'')


class MachineSysEvents(Base):
    __tablename__ = 'machine_sys_events'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    id = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    client_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    machine_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    machine_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    event_type = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    action = Column(types.Unicode(45), nullable=True, unique=None, default=u'')
    description = Column(types.UnicodeText(), nullable=True, unique=None, default=None)
    add_time = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    add_time_int = Column(types.Integer(), nullable=True, unique=None, default=0)


class Users(Base):
    __tablename__ = 'users'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    conn_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    user_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    role = Column(types.Unicode(1000), nullable=True, unique=None, default=u'')
    name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    mail_address = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    cell_phone = Column(types.Unicode(20), nullable=True, unique=None, default=u'')
    company = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    notes = Column(types.Unicode(1024), nullable=True, unique=None, default=u'')
    state = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    all_kiosks = Column(types.Integer(), nullable=True, unique=None, default=0)
    alert_same_kiosk = Column(types.Integer(), nullable=True, unique=None, default=1)
    alert_all_kiosk = Column(types.Integer(), nullable=True, unique=None, default=0)
    verify_code = Column(types.Unicode(20), nullable=True, unique=None, default=u'')
    add_time = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    last_update_time = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    add_time_int = Column(types.Integer(), nullable=True, unique=None, default=0)
    last_update_time_int = Column(types.Integer(), nullable=True, unique=None, default=0)


class UserKiosk(Base):
    __tablename__ = 'user_kiosk'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    user_email = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    machine_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    type = Column(types.Integer(), nullable=True, unique=None, default=1)


class UserEvents(Base):
    __tablename__ = 'user_events'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    host_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    host_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    client_id = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    client_name = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    email = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    ip = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    category = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    sub_category = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    action = Column(types.Unicode(50), nullable=True, unique=None, default=u'')
    action_level = Column(types.Unicode(20), nullable=True, unique=None, default=u'')
    data1 = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    data2 = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    data3 = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    data4 = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    data5 = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    add_time = Column(types.DateTime(), nullable=True, unique=None, default=u'0000-00-00 00:00:00')
    add_time_int = Column(types.Integer(), nullable=True, unique=None, default=0)


class UserRole(Base):
    __tablename__ = 'user_role'
    id_ai = Column(types.Integer(), nullable=False, unique=None, default=None, primary_key=True, autoincrement=True)
    email = Column(types.Unicode(255), nullable=True, unique=None, default=u'')
    role_id = Column(types.Unicode(50), nullable=True, unique=None, default=u'')



from services.user.user import *
from system.test import *


@fixture
def service(app):
    service = app.create(UserService)
    service.add_user(conn_id='conn_id',
                     user_name='user_name',
                     mail_address='mail_address',
                     role='role',
                     name='name',
                     cell_phone='cell_phone',
                     company='company',
                     all_kiosks=0,
                     alert_same_kiosk=0,
                     alert_all_kiosk=0)
    return service


def test_add_user(service):
    user = service.db.query(Users).first()
    assert user.conn_id == 'conn_id'
    assert user.user_name == 'user_name'
    assert user.mail_address == 'mail_address'
    assert user.role == 'role'


def test_update_users_conn_id(service):
    service.update_users_conn_id('new_conn_id', 'conn_id')
    user = service.db.query(Users).first()
    assert user.conn_id == 'new_conn_id'


def test_get_user_info_by_email(service):
    user = service.get_user_info_by_email('mail_address')
    assert user["mail_address"] == 'mail_address'


def test_get_users(service):
    users = service.get_users('conn_id')
    assert users[0]['conn_id'] == 'conn_id'


def test_get_users_count(service):
    count = service.get_users_count('conn_id')
    assert count == 1



from nameko.rpc import rpc, RpcProxy
from nameko.events import EventDispatcher, event_handler
from dao import *


class App(object):

    """
    Supply database and datasets.

    Returns
    -------
    - db : sqlalchemy.orm.session.Session
    - rs : redis.client.StrictRedis

    Examples
    --------
    Create a new application.

    >>> from system.app import App
    >>> app = App()
    >>> app.db
    ''
    >>> app.rs
    ''
    """

    db = ""
    rs = ""

    def init(self):
        data = Db()
        self.db = data.get_db()
        self.rs = data.get_rs()

    def obj2dict(self, obj):
        return dict((key, obj.__dict__[key])\
               for key in obj.__dict__ \
               if not key.startswith("_"))



from dao import *
import mongomock
from mock import *
from pytest import *
from mockredis import *

class Mock_app(object):
    """
    Platform App for test

    Examples
    --------
    >>> from system.test import *
    >>> app = Mock_app()
    """

    app_base = "system.app.App."
    log_base = "system.log.Log."
    log_method = "get"
    app_methods = ["rs", "db", "init"]


    def mongo(self):
        return mongomock.MongoClient().db.collection

    def rs(self):
        return mock_redis_client()

    def db(self):
        engine = create_engine('sqlite:///:memory:')
        Base.metadata.create_all(engine)
        return sessionmaker(engine)()

    def init(self):
        return lambda self: None

    def create(self, name):
        self.service = name()
        return self.service

    def get_attr(self, attr):
        attr.side_effect = self.get_effect

    def get_effect(self, args):
        self.get_result = args
        
    def set_attr(self, attr, value):
        attr.return_value = value

    def start(self, patch):
        methods = [self.rs, self.db, self.init]
        methods_zip = zip(self.app_methods, methods)
        for app_method, method in methods_zip:
            patch.setattr(self.app_base + app_method, method())
        patch.setattr(self.log_base + self.log_method, self.mongo)


@fixture
def app(monkeypatch):
    """
    Create platform App for test
    """

    mock_app = Mock_app()
    mock_app.start(monkeypatch)

    return mock_app



from services.log.log import *
from system.test import *


@fixture
def service(app):
    service = app.create(LogService)
    return service


def test_add_log(service):
    log = {"content": "info"}
    service.add_log(log)
    result = service.log.find_one(log)
    assert result["content"] == "info" 



from system.app import *
import time
from datetime import date, datetime, timedelta


class UserService(App):

    """
    Support all user operations.

    >>> from nameko.standalone.rpc import ClusterRpcProxy
    >>> CONFIG = {"AMQP_URI":"amqp://guest:guest@localhost"}
    >>> with ClusterRpcProxy(CONFIG) as services:
    >>>     result = services.user_service.add_user(conn_id='10', 
    >>>              user_name='', mail_address="x@y", role='')
    """

    name = "user_service"

    def __init__(self):
        super(UserService, self).init()

    @rpc
    def add_user(self, conn_id, user_name, mail_address, 
                 role, name=None, cell_phone='', company='', 
                 all_kiosks=0, alert_same_kiosk=0, 
                 alert_all_kiosk=0):
        """ Add a user. """
        
        current_date = datetime.utcnow()
        current_time = int(time.time())

        user = Users()
        user.conn_id = conn_id
        user.user_name = user_name or mail_address
        user.role = role
        user.name = name
        user.mail_address = mail_address
        user.state = "active"
        user.cell_phone = cell_phone
        user.company = company
        user.all_kiosks = all_kiosks
        user.alert_same_kiosk = alert_same_kiosk
        user.alert_all_kiosk = alert_all_kiosk
        user.add_time = current_date
        user.add_time_int = current_time
        user.last_update_time = current_date
        user.last_update_time_int = current_time

        self.db.add(user)
        self.db.commit()

    @rpc
    def update_users_conn_id(self, conn_id, old_conn_id):
        """ Update conn_id """

        self.db.query(Users).\
                filter(Users.conn_id == old_conn_id).\
                update({Users.conn_id: conn_id})

    @rpc
    def get_user_info_by_email(self, email):
        """ return user info by email """

        user = self.db.query(Users).\
                filter(Users.mail_address == email).\
                first()
        return self.obj2dict(user)

    @rpc
    def get_users(self, conn_id, current_user_role=None, 
            search_key=None, sort_key=None, sort_order=None, 
            limit=None, offset=None):
        """ get the users by conn ID """

        rows = self.db.query(Users).filter(Users.conn_id == conn_id)
        if current_user_role == "operator":
            return []

        if current_user_role == "admin":
            rows = rows.filter(Users.role != "sadmin")

        if search_key:
            key = "%%%s%%" % search_key
            rows = rows.filter(or_(Users.user_name.like(key),
                Users.name.like(key), Users.mail_address.like(key)))

        if sort_key:
            sort = getattr(Users, sort_key, None)
            if sort is not None:
                if str(sort_order).lower() == "desc":
                    rows = rows.order_by(sort.desc())
                else:
                    rows = rows.order_by(sort)
        if limit:
            rows = rows.limit(limit)

        if offset:
            rows = rows.offset(offset)

        result = rows.all()

        if sort_key:
            if sort_key == "role":
                if str(sort_order).lower() == "desc":
                    # to compare the index of ('admin')
                    result.sort(
                        lambda x, y: cmp(x.role.find("admin"), y.role.find("admin")))
                else:
                    result.sort(
                        lambda x, y: cmp(y.role.find("admin"), x.role.find("admin")))
        return [ self.obj2dict(user) for user in result ] if result else []

    @rpc
    def get_users_count(self, conn_id, current_user_role=None, search_key=None):
        """ get the users by conn ID """

        rows = self.db.query(Users).filter(Users.conn_id == conn_id)
        if current_user_role == "operator":
            return 0
        if current_user_role == "admin":
            rows = rows.filter(Users.role != "sadmin")

        if search_key:
            key = "%%%s%%" % search_key
            rows = rows.filter(or_(Users.user_name.like(key),
                                   Users.name.like(key),
                                   Users.mail_address.like(key)))

        return rows.count()

    @rpc
    def set_user_info(self, email, username=None, role=None,
                      notes=None, state=None, name=None,
                      cell_phone=None, company=None, all_kiosks=None,
                      same_assign=None, alert_all=None):
        """ edit the userinfo """

        user = self.db.query(Users).filter(
            Users.mail_address == email).first()
        if username is not None:
            user.username = username
        if role is not None:
            user.role = role
        if notes is not None:
            user.notes = notes
        if state is not None:
            user.state = state
        if name is not None:
            user.name = name
        if same_assign is not None:
            user.alert_same_kiosk = same_assign
        if alert_all is not None:
            user.alert_all_kiosk = alert_all
        if cell_phone:
            user.cell_phone = cell_phone
        if company:
            user.company = company
        if all_kiosks is not None:
            user.all_kiosks = all_kiosks
        user.last_update_time = datetime_utils.current_time()
        user.last_update_time_int = datetime_utils.get_interger_time()
        self.db.flush()

    @rpc
    def delete_user(self, email):
        """ Delete user """

        self.db.query(Users).filter(Users.mail_address == email).delete()
        self.db.flush()

    @rpc
    def reactivate_user(self, user_email):
        """ Reactivate user """

        user = self.db.query(Users).filter(
            Users.mail_address == user_email).first()
        user.state = 'active'
        self.db.flush()

    @rpc
    def get_user_role_msg(self, email):
        """ Get user's role """

        user_info = self.db.query(Users).filter(
            Users.mail_address == email).first()
        if user_info:
            return user_info.role
        else:
            return ""

    @rpc
    def save_user_role_msg(self, email, roles):
        """ Save user's role """

        user_info = self.db.query(Users).filter(
            Users.mail_address == email).first()
        if user_info:
            user_info.role = roles
        self.db.flush()

    @rpc
    def get_client_user_list_count(self, conn_id, search_key=None,
                                   role=None, state=None):
        """ Get user list count """

        sql = self.db.query(Users).filter(Users.conn_id == conn_id)
        if role:
            sql = sql.filter(Users.role == role)
        if state:
            sql = sql.filter(Users.state == state)
        if search_key:
            key = "%%%s%%" % search_key
            sql = sql.filter(or_(Users.user_name.like(key),
                                 Users.name.like(key),
                                 Users.notes.like(key)))
        return sql.count()

    @rpc
    def delete_user_by_conn_id(self, conn_id):
        """ Delete user """

        self.db.query(Users).filter(Users.conn_id == conn_id).delete()



#!/usr/bin/env python

import os
import sys
from codecs import open
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.txt'), 'r', 'utf-8') as handle:
    long_description = handle.read()

requires = [
    "alabaster==0.7.9",
    "amqp==2.1.1",
    "Babel==2.3.4",
    "cffi==1.9.1",
    "colorama==0.3.7",
    "cryptography==1.7.1",
    "docopt==0.6.2",
    "docutils==0.12",
    "ecdsa==0.13",
    "eventlet==0.19.0",
    "Fabric3==1.13.1.post1",
    "funcsigs==1.0.2",
    "futures==3.0.5",
    "gevent==1.1.2",
    "greenlet==0.4.10",
    "idna==2.1",
    "imagesize==0.7.1",
    "Jinja2==2.8",
    "kombu==4.0.0",
    "MarkupSafe==0.23",
    "mock==2.0.0",
    "mockredispy==2.9.3",
    "mocksqlalchemy==0.1.2",
    "mogo==0.4.0",
    "mongomock==3.7.0",
    "mysqlclient==1.3.9",
    "nameko==2.4.2",
    "nameko-sqlalchemy==0.0.4",
    "paramiko==1.17.2",
    "path.py==9.0",
    "pbr==1.10.0",
    "pluggy==0.4.0",
    "py==1.4.31",
    "pyasn1==0.1.9",
    "pycparser==2.17",
    "pycrypto==2.6.1",
    "Pygments==2.1.3",
    "pymongo==3.3.1",
    "PyMySQL==0.7.9",
    "pytest==3.0.4",
    "pytz==2016.7",
    "PyYAML==3.12",
    "redis==2.10.5",
    "requests==2.12.3",
    "sentinels==1.0.0",
    "six==1.10.0",
    "snowballstemmer==1.2.1",
    "Sphinx==1.5b1",
    "SQLAlchemy==1.1.4",
    "vine==1.1.3",
    "Werkzeug==0.11.11",
]

setup(
    name='service-platform',
    version='1.6',
    author='Eric.Zhou',
    author_email='xiaoxiang.cn@gmail.com',
    url='https://github.com/zhouxiaoxiang/platform',
    long_description=long_description,
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    description='Cereson Platform.', )



import logging
import mogo
from system.conf import Config


class Log(object):
    """ 
    Create log handler 

    Modules
    -------
    - sysLog
    - sysEventLog
    - userEventLog 
    - stockEventLog
    - nopickedEventLog

    Returns
    -------
    Collection

    Examples
    --------
    >>> from system.log import Log
    >>> log = Log("module").get()
    >>> result = log.insert_one({"count":1})
    >>> result.acknowledged
    True
    """ 

    def __init__(self, module=""):
        """ Require server info """

        conf = Config()['log'][module]
        self.host = conf['host']
        self.db = conf['db']
        self.tb = conf['tb']

    def get(self):
        """ Return log handler """

        self.conn = mogo.connect(self.db, self.host)
        self.log = self.conn[self.db][self.tb]
        return self.log
